<?php

/*
 * This file is part of the Qsnh/meedu.
 *
 * (c) XiaoTeng <616896861@qq.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace Addons\No9Theme\Http\Controllers\Api;

use App\Http\Controllers\Api\V2\BaseController;
use Illuminate\Http\Request;
use App\Constant\ApiV2Constant;
use App\Businesses\BusinessState;
use App\Http\Requests\ApiV2\CommentRequest;
use App\Services\Base\Services\ConfigService;
use App\Services\Member\Services\UserService;
use App\Services\Order\Services\OrderService;
use App\Services\Course\Services\VideoService;
use App\Services\Course\Services\CourseService;
use App\Services\Course\Services\VideoCommentService;
use App\Services\Base\Interfaces\ConfigServiceInterface;
use App\Services\Member\Interfaces\UserServiceInterface;
use App\Services\Order\Interfaces\OrderServiceInterface;
use App\Services\Course\Interfaces\VideoServiceInterface;
use App\Services\Course\Interfaces\CourseServiceInterface;
use App\Services\Course\Interfaces\VideoCommentServiceInterface;

/**
 * @OpenApi\Annotations\Schemas(
 *     @OA\Schema(
 *         schema="PlayUrlItem",
 *         type="object",
 *         title="视频播放地址",
 *         @OA\Property(property="url",type="string",description="视频播放地址"),
 *         @OA\Property(property="format",type="string",description="视频格式"),
 *         @OA\Property(property="duration",type="integer",description="时长，秒"),
 *         @OA\Property(property="name",type="string",description="码率"),
 *     ),
 * )
 */

/**
 * Class VideoController
 * @package App\Http\Controllers\Api\V2
 *
 */
class VideoController extends BaseController
{

    /**
     * @var VideoService
     */
    protected $videoService;
    /**
     * @var ConfigService
     */
    protected $configService;
    /**
     * @var VideoCommentService
     */
    protected $videoCommentService;
    /**
     * @var UserService
     */
    protected $userService;
    /**
     * @var CourseService
     */
    protected $courseService;
    /**
     * @var BusinessState
     */
    protected $businessState;
    /**
     * @var OrderService
     */
    protected $orderService;

    public function __construct(
        VideoServiceInterface $videoService,
        ConfigServiceInterface $configService,
        VideoCommentServiceInterface $videoCommentService,
        UserServiceInterface $userService,
        CourseServiceInterface $courseService,
        BusinessState $businessState,
        OrderServiceInterface $orderService
    )
    {
        $this->videoService = $videoService;
        $this->configService = $configService;
        $this->videoCommentService = $videoCommentService;
        $this->userService = $userService;
        $this->courseService = $courseService;
        $this->businessState = $businessState;
        $this->orderService = $orderService;
    }

    public function show(Request $request, $courseId, $id, $slug)
    {
        $scene = $request->input('scene');
        $course = $this->courseService->find($courseId);
        $video = $this->videoService->find($id);

        // 视频浏览次数
        $this->videoService->viewNumInc($video['id']);

        // 视频评论
        $comments = $this->videoCommentService->videoComments($video['id']);
        $commentUsers = $this->userService->getList(array_column($comments, 'user_id'), ['role']);
        $commentUsers = array_column($commentUsers, null, 'id');

        // 课程章节
        $chapters = $this->courseService->chapters($video['course_id']);
        $videos = $this->videoService->courseVideos($video['course_id']);

        // 是否可以观看视频
        $canSeeVideo = false;
        // 试看
        $trySee = false;
        // 课程视频观看进度
        $videoWatchedProgress = [];
        // 是否可以评论
        $canComment = $this->businessState->videoCanComment($this->user(), $video);

        if ($this->check()) {
            $canSeeVideo = $this->businessState->canSeeVideo($this->user(), $video['course'], $video);
            $canSeeVideo && $this->courseService->recordUserCount($this->id(), $course['id']);

            $userVideoWatchRecords = $this->userService->getUserVideoWatchRecords($this->id(), $course['id']);
            $videoWatchedProgress = array_column($userVideoWatchRecords, null, 'video_id');

            $trySee = $canSeeVideo === false && $video['free_seconds'] > 0;
        }

        // 下一个视频
        $nextVideo = call_user_func(function () use ($chapters, $videos, $video) {
            $nextVideo = null;
            $index = false;
            $lock = false;
            foreach ($chapters ?: [['id' => 0]] as $chapter) {
                $chapterId = $chapter['id'];
                $items = $videos[$chapterId] ?? [];
                if (!$items) {
                    continue;
                }
                if ($index === false && $chapterId !== $video['chapter_id']) {
                    continue;
                }
                $index = true;
                foreach ($items as $item) {
                    if ($lock === false && $item['id'] !== $video['id']) {
                        continue;
                    }
                    if ($lock === true) {
                        $nextVideo = $item;
                        break 2;
                    }
                    $lock = true;
                }
            }
            return $nextVideo;
        });

        // 播放地址
        $playUrls = collect([]);
        if (!($video['aliyun_video_id'] && $this->configService->getAliyunPrivatePlayStatus())) {
            $playUrls = get_play_url($video, $trySee);
            if ($playUrls->isEmpty()) {
                flash('没有播放地址');
                return back();
            }
        }

        $title = $video['title'];
        $keywords = $video['seo_keywords'];
        $description = $video['seo_description'];

        return $this->data(compact(
            'course',
            'video',
            'title',
            'keywords',
            'description',
            'comments',
            'commentUsers',
            'videos',
            'chapters',
            'canSeeVideo',
            'scene',
            'playUrls',
            'nextVideo',
            'videoWatchedProgress',
            'trySee',
            'canComment'
        ));
    }
}
